--[[
Prescription - Copyright (C) 2007 Michael Joseph Murray aka Starinnia of Lothar
contact: codemaster2010@gmail.com

-----------------------------------------------------------------------------------------
This addon is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.0 of the License, or (at your option) any later version.

This addon is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
]]

Prescription = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceHook-2.1")

Prescription.Battle = {
	"Healing Power", -- Elixir of Healing Power
	"Major Agility", -- Elixir of Major Agility
	"Major Firepower", -- Elixir of Major Firepower
	"Major Shadow Power", -- Elixir of Major Shadow Power
	"Major Strength", -- Elixir of Major Strength
	"Elixir of Mastery",
	"Major Frost Power", -- Elixir of Major Frost Power
	"Fel Strength Elixir",
	"Greater Arcane Elixir",
	"Onslaught Elixir",
	"Adept's Elixir",
}

Prescription.Guardian = {
	"Elixir of Draenic Wisdom",
	"Empowerment", -- Elixir of Empowerment
	"Elixir of Ironskin",
	"Major Armor", -- Elixir of Major Defense
	"Elixir of Major Fortitude",
	"Greater Mana Regeneration", -- Elixir of Major Mageblood
}

Prescription.Flask = {
	"Flask of Chromatic Resistance",
	"Flask of Distilled Wisdom",
	"Flask of Fortification",
	"Flask of Mighty Restoration",
	"Flask of Relentless Assault",
	"Flask of Supreme Power",
	"Flask of Arcane Fortification",
	"Flask of Shadow Fortification",
	---      Shattrath Flasks     ---
	"Relentless Assault of Shattrath",
	"Mighty Restoration of Shattrath",
	"Fortification of Shattrath",
	"Supreme Power of Shattrath",
}

Prescription.Food = {
	"Well Fed",
}

local user = UnitName("player")

function Prescription:OnInitialize()
	local defaults = {
		checkBattle = true,
		checkGuardian = true,
		checkFood = true,
		checkFlask = true,
		sendWhispers = true,
		sendRaid = false,
		blockOutgoing = false,
		blackString = "",
		blacklist = {}
	}
	
	Prescription:RegisterDB("PrescriptionDB")
	Prescription:RegisterDefaults('profile', defaults)
	
	local opts = {
		type = 'group',
		args = {
			check = {
				type = 'execute',
				name = "Check Consumables",
				desc = "Check the raid for consumable usage",
				func = "Raid_Check",
			},
			soul = {
				type = 'execute',
				name = "Check for Soul Stone",
				desc = "Check the raid for who has Soul Stones",
				func = "SS_Check",
			},
			blacklist = {
				type = 'text',
				name = "Blacklist Raiders",
				desc = "Add Raiders to the Blacklist",
				usage = "<player to blacklist>",
				get = "getBlackList",
				set = function(v) table.insert(self.db.profile.blacklist, v); self:serializeBlackList(); end,
			},
			viewlist = {
				type = 'execute',
				name = "View Blacklist",
				desc = "Print out the Blacklisted toons",
				func = "blackListDump",
			},
			clearlist = {
				type = 'execute',
				name = "Clear Blacklist",
				desc = "Remove all names from blacklist",
				func = "clearBlackList",
			},
			drugs = {
				type = 'group',
				name = "Prescription Drugs",
				desc = "What do you want to check for?",
				args = {
					battle = {
						type = 'toggle',
						name = "Battle Elixirs",
						desc = "Toggle checking for Battle Elixirs",
						get = function() return self.db.profile.checkBattle end,
						set = function() self.db.profile.checkBattle = not self.db.profile.checkBattle end,
					},
					guardian = {
						type = 'toggle',
						name = "Guardian Elixirs",
						desc = "Toggle checking for Guardian Elixirs",
						get = function() return self.db.profile.checkGuardian end,
						set = function() self.db.profile.checkGuardian = not self.db.profile.checkGuardian end,
					},
					flask = {
						type = 'toggle',
						name = "Flasks",
						desc = "Toggle checking for Flasks",
						get = function() return self.db.profile.checkFlask end,
						set = function() self.db.profile.checkFlask = not self.db.profile.checkFlask end,
					},
					food = {
						type = 'toggle',
						name = "Food Buffs",
						desc = "Toggle checking for Food Buffs",
						get = function() return self.db.profile.checkFood end,
						set = function() self.db.profile.checkFood = not self.db.profile.checkFood end,
					},
				},
			},
			announce = {
				type = 'group',
				name = "Announcements",
				desc = "How to announce Prescriptions",
				args = {
					whisper = {
						type = 'toggle',
						name = "Whisper Players",
						desc = "Whisper players missing consumables",
						get = function() return self.db.profile.sendWhispers end,
						set = function() self.db.profile.sendWhispers = not self.db.profile.sendWhispers end,
					},
					blockouts = {
						type = 'toggle',
						name = "Block Outgoing Whispers",
						desc = "Toggles blocking of outgoing <Rx> whispers",
						get = function() return self.db.profile.blockOutgoing end,
						set = function() self.db.profile.blockOutgoing = not self.db.profile.blockOutgoing end,
					},
					raid = {
						type = 'toggle',
						name = "Raid Report",
						desc = "Send consumable reports to raid chat",
						get = function() return self.db.profile.sendRaid end,
						set = function() self.db.profile.sendRaid = not self.db.profile.sendRaid end,
					},
				},
			},
		},
	}
	
	Prescription:RegisterChatCommand("/prescription", "/rx", opts)
	
	self:Hook(DEFAULT_CHAT_FRAME, "AddMessage", true)
	self:Hook(ChatFrame2, "AddMessage", true)
	self:Hook(ChatFrame3, "AddMessage", true)
	self:Hook(ChatFrame4, "AddMessage", true)
	self:Hook(ChatFrame5, "AddMessage", true)
	self:Hook(ChatFrame6, "AddMessage", true)
	self:Hook(ChatFrame7, "AddMessage", true)
end

function Prescription:AddMessage(frame, text, r, g, b, id)
	self.search = string.find(text, "<Rx>: ")
	if self.search and id == 8 and self.db.profile.blockOutgoing then
		text = ""
		self.hooks[frame].AddMessage(frame, text, r, g, b, id)
		self.search = nil
	else
		self.hooks[frame].AddMessage(frame, text, r, g, b, id)
	end
end

function Prescription:serializeBlackList()
	for k, v in pairs(self.db.profile.blacklist) do
		self.db.profile.blackString = self.db.profile.blackString .. v .. ","
	end
end

function Prescription:blackListDump()
	for k, v in pairs(self.db.profile.blacklist) do
		self:Print(v)
	end
end

function Prescription:getBlackList()
	local holder = ""
	for k, v in pairs(self.db.profile.blacklist) do
		holder = holder .. v .. ", "
	end
	return holder
end

function Prescription:clearBlackList()
	self.db.profile.blacklist = {}
	self:Print("Blacklist Cleared!")
end

function Prescription:SS_Check()
	if GetNumRaidMembers() == 0 then
		self:Print("|cffff0000You are not in a raid")
		return
	end
	
	local raid = {}
	for i = 1, GetNumRaidMembers() do
		local n = 1
		if UnitBuff("raid"..i, n) == nil then
			if not raid[i] then raid[i] = {} end
			table.insert(raid[i], "No Buffs")
		end
		while UnitBuff("raid"..i, n) ~= nil do
			local name, rank, tex, count, duration, timeLeft = UnitBuff("raid"..i, n)
			if not raid[i] then raid[i] = {} end
			table.insert(raid[i], name)
			n = n + 1
		end
	end
	
	local ssFound = false
	for i = 1, GetNumRaidMembers() do
		local toon, realm = UnitName("raid"..i)
		for j = 1, 24 do
			local temp = table.remove(raid[i])
			if temp == nil then
				break
			elseif temp == "No Buffs" then
				break
			elseif temp == "Soulstone Resurrection" then
				ssFound = true
				self:Print(toon .. " has a soul stone.")
			end
		end
	end
	
	if not ssFound then
		self:Print("No soul stone was found.")
	end
	
end

function Prescription:Raid_Check()	
	if GetNumRaidMembers() == 0 then
		self:Print("|cffff0000You are not in a raid")
		return
	end
	
	local raid = {}
	for i = 1, GetNumRaidMembers() do
		local n = 1
		if UnitBuff("raid"..i, n) == nil then
			if not raid[i] then raid[i] = {} end
			table.insert(raid[i], "No Buffs")
		end
		while UnitBuff("raid"..i, n) ~= nil do
			local name, rank, tex, count, duration, timeLeft = UnitBuff("raid"..i, n)
			if not raid[i] then raid[i] = {} end
			table.insert(raid[i], name)
			n = n + 1
		end
	end
	
	for i = 1, GetNumRaidMembers() do
		local e1, e2, f1, f2 = false, false, false, false
		local toon, realm = UnitName("raid"..i)
		self.match = string.find(self.db.profile.blackString, toon)
		if self.match  then
			self:Print(toon .. " has been blacklisted.")
			self.match = nil
			break
		else
			for j = 1, 24 do
				local temp = table.remove(raid[i])
				if temp == nil then
					break
				elseif temp == "No Buffs" then
					break
				else
					if self.db.profile.checkBattle then
						for k, v in pairs(self.Battle) do
							if temp == v then
								e1 = true
							end
						end
					else
						e1 = true
					end
					if self.db.profile.checkGuardian then
						for k, v in pairs(self.Guardian) do
							if temp == v then
								e2 = true
							end
						end
					else
						e2 = true
					end
					if self.db.profile.checkFlask then
						for k, v in pairs(self.Flask) do
							if temp == v then
								f1 = true
							end
						end
					else
						f1 = true
					end
					if self.db.profile.checkFood then
						for k, v in pairs(self.Food) do
							if temp == v then
								f2 = true
							end
						end
					else
						f2 = true
					end
				end
			end
			
			--Local message stubs
			local message = toon .. ": "
			local raid = "<Rx> " .. toon .. " is using: "
			local whisper = "<Rx> You are missing: "
			
			--Logic for printing missing buffs
			if e1 then
				message = message .. "Battle, "
				raid = raid .. "Battle, "
			end
			if e2 then
				message = message .. "Guardian, "
				raid = raid .. "Guardian, "
			end
			if f1 then
				message = message .. "Flask, "
				raid = raid .. "Flask, "
			end
			if f2 then
				message = message .. "Food"
				raid = raid .. "Food"
			end
			if (not e1) and (not e2) and (not f1) and (not f2) then
				message = message .. "No Consumables Found"
				raid = raid .. "No Consumables!"
			end
			
			-- Flask fix for whispering missing buffs
			if f1 then
				e1, e2 = true, true
			end
			
			--Logic for whispering missing buffs
			if not e1 then
				whisper = whisper .. "Battle elixir, "
			end
			if not e2 then
				whisper = whisper .. "Guardian elixir, "
			end
			if (not f1) and (not e1) and (not e2) then
				whisper = whisper .. "Flask, "
			end
			if not f2 then
				whisper = whisper .. "Food"
			end
			
			--Send the messages
			self:Print(message)
			if self.db.profile.sendRaid then
				SendChatMessage(raid, "RAID")
			end
			if self.db.profile.sendWhispers and toon ~= user then
				SendChatMessage(whisper, "WHISPER", nil, toon)
			end
		end
	end
end
